@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: idled.c,v 1.28 2010/01/06 17:01:32 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -64,9 +62,6 @@
 #include "hash.h"
 #include "exitcodes.h"
 
-/* global state */
-const int config_need_data = 0;
-
 extern int optind;
 extern char *optarg;
 
@@ -81,7 +76,7 @@ struct ientry {
 };
 static struct hash_table itable;
 
-void fatal(const char *msg, int err)
+EXPORTED void fatal(const char *msg, int err)
 {
     if (debugmode) fprintf(stderr, "dying with %s %d\n",msg,err);
     syslog(LOG_CRIT, "%s", msg);
@@ -94,9 +89,9 @@ void fatal(const char *msg, int err)
 
 static int mbox_count_cb(void *rockp,
 			 const char *key __attribute__((unused)),
-			 int keylen __attribute__((unused)),
+			 size_t keylen __attribute__((unused)),
 			 const char *data __attribute__((unused)),
-			 int datalen __attribute__((unused)))
+			 size_t datalen __attribute__((unused)))
 {
     int *ip = (int *) rockp;
     (*ip)++;
@@ -296,7 +291,7 @@ int main(int argc, char **argv)
 	}
     }
 
-    cyrus_init(alt_config, "idled", 0);
+    cyrus_init(alt_config, "idled", 0, 0);
 
     /* Set inactivity timer (convert from minutes to seconds) */
     idle_timeout = config_getint(IMAPOPT_TIMEOUT);
@@ -306,7 +301,7 @@ int main(int argc, char **argv)
     /* count the number of mailboxes */
     mboxlist_init(0);
     mboxlist_open(NULL);
-    mboxlist_allmbox("", &mbox_count_cb, &nmbox);
+    mboxlist_allmbox("", &mbox_count_cb, &nmbox, /*incdel*/0);
     mboxlist_close();
     mboxlist_done();
 
@@ -348,8 +343,6 @@ int main(int argc, char **argv)
     for (;;) {
 	int n;
 
-	signals_poll();
-
 	/* check for shutdown file */
 	if (shutdown_file(NULL, 0)) {
 	    /* signal all processes to shutdown */
@@ -389,10 +382,3 @@ int main(int argc, char **argv)
     shut_down(1);
 }
 
-void printstring(const char *s __attribute__((unused)))
-{ 
-    /* needed to link against annotate.o */
-    fatal("printstring() executed, but its not used for POP3!",
-          EC_SOFTWARE);
-}
-
