@@ -485,8 +485,7 @@ EXPORTED int mboxlist_lookup(const char *name, mbentry_t **entryptr,
     return 0;
 }
 
-/* now this is just silly, but hey */
-EXPORTED int mboxlist_lookup_allow_reserved(const char *name,
+EXPORTED int mboxlist_lookup_allow_all(const char *name,
 				   mbentry_t **entryptr,
 				   struct txn **tid)
 {
@@ -1103,6 +1102,7 @@ mboxlist_delayed_deletemailbox(const char *name, int isadmin,
 			       struct auth_state *auth_state,
 			       struct mboxevent *mboxevent,
 			       int checkacl,
+			       int localonly,
 			       int force)
 {
     mbentry_t *mbentry = NULL;
@@ -1162,6 +1162,7 @@ mboxlist_delayed_deletemailbox(const char *name, int isadmin,
                                1 /* isadmin */, userid,
                                auth_state,
 			       mboxevent,
+			       localonly /* local_only */,
                                force, 1);
 
     mboxlist_entry_free(&mbentry);
@@ -1215,12 +1216,9 @@ EXPORTED int mboxlist_deletemailbox(const char *name, int isadmin,
 	if (!isadmin) { r = IMAP_PERMISSION_DENIED; goto done; }
     }
 
-    r = mboxlist_lookup(name, &mbentry, NULL);
+    r = mboxlist_lookup_allow_all(name, &mbentry, NULL);
     if (r) goto done;
 
-    if (mbentry->mbtype & MBTYPE_DELETED)
-	goto done;
-
     isremote = mbentry->mbtype & MBTYPE_REMOTE;
 
     /* check if user has Delete right (we've already excluded non-admins
@@ -1321,7 +1319,7 @@ EXPORTED int mboxlist_renamemailbox(const char *oldname, const char *newname,
 			   int isadmin, const char *userid,
 			   struct auth_state *auth_state,
 			   struct mboxevent *mboxevent,
-			   int forceuser, int ignorequota)
+			   int local_only, int forceuser, int ignorequota)
 {
     int r;
     long myrights;
@@ -1512,7 +1510,7 @@ EXPORTED int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	goto done;
     }
 
-    if (config_mupdate_server) {
+    if (!local_only && config_mupdate_server) {
 	/* commit the mailbox in MUPDATE */
 	char *loc = strconcat(config_servername, "!", newpartition, (char *)NULL);
 
@@ -1554,6 +1552,7 @@ EXPORTED int mboxlist_renamemailbox(const char *oldname, const char *newname,
 		    mboxevent_extract_mailbox(mboxevent, newmailbox);
 		    mboxevent_extract_old_mailbox(mboxevent, oldmailbox);
 		}
+
 		mboxevent_set_access(mboxevent, NULL, NULL, userid, newmailbox->name, 1);
 	    }
 
@@ -1862,7 +1861,7 @@ EXPORTED int mboxlist_setacl(struct namespace *namespace, const char *name,
 	struct mboxevent *mboxevent = mboxevent_new(EVENT_ACL_CHANGE);
 	mboxevent_extract_mailbox(mboxevent, mailbox);
 	mboxevent_set_acl(mboxevent, identifier, rights);
-	mboxevent_set_access(mboxevent, NULL, NULL, userid, mailbox->name, 1);
+	mboxevent_set_access(mboxevent, NULL, NULL, userid, mailbox->name, 0);
 
 	mboxevent_notify(mboxevent);
 	mboxevent_free(&mboxevent);
@@ -2838,10 +2837,15 @@ EXPORTED int mboxlist_setquotas(const char *root,
 	    if (q.limits[res] != newquotas[res]) {
 		underquota = 0;
 
-		/* prepare a QuotaChange event notification */
-		if (quotachange_event == NULL)
+		/* Prepare a QuotaChange event notification *now*.
+		 * 
+		 * This is to ensure the QuotaChange is emitted before the
+		 * subsequent QuotaWithin (if the latter becomes applicable).
+		 */
+		if (quotachange_event == NULL) {
 		    quotachange_event = mboxevent_enqueue(EVENT_QUOTA_CHANGE,
 		                                          &mboxevents);
+		}
 
 		/* prepare a QuotaWithin event notification if now under quota */
 		if (quota_is_overquota(&q, res, NULL) &&
@@ -2860,10 +2864,21 @@ EXPORTED int mboxlist_setquotas(const char *root,
 		    mboxevent_extract_quota(quotawithin_event, &q, res);
 	    }
 	}
-	if (changed)
+	if (changed) {
 	    r = quota_write(&q, &tid);
+
+	    if (quotachange_event == NULL) {
+		quotachange_event = mboxevent_enqueue(EVENT_QUOTA_CHANGE, &mboxevents);
+	    }
+
+	    for (res = 0; res < QUOTA_NUMRESOURCES; res++) {
+		mboxevent_extract_quota(quotachange_event, &q, res);
+	    }
+	}
+
 	if (!r)
 	    quota_commit(&tid);
+
 	goto done;
     }
 
@@ -2916,10 +2931,11 @@ EXPORTED int mboxlist_setquotas(const char *root,
     if (r) goto done;
 
     /* prepare a QuotaChange event notification */
-    quotachange_event = mboxevent_enqueue(EVENT_QUOTA_CHANGE, &mboxevents);
-    for (res = 0 ; res < QUOTA_NUMRESOURCES ; res++) {
-    	if (q.limits[res] != QUOTA_UNLIMITED)
-    	    mboxevent_extract_quota(quotachange_event, &q, res);
+    if (quotachange_event == NULL)
+	quotachange_event = mboxevent_enqueue(EVENT_QUOTA_CHANGE, &mboxevents);
+
+    for (res = 0; res < QUOTA_NUMRESOURCES; res++) {
+	mboxevent_extract_quota(quotachange_event, &q, res);
     }
 
     quota_commit(&tid);
