@@ -80,7 +80,7 @@ my %builtins = (exit =>
 		listaclmailbox => 'listacl',
 		lm => 'listmailbox',
 		listmailbox =>
-		  [\&_sc_list, '[-subscribed] [pattern [base]]',
+		  [\&_sc_list, '[-subscribed] [-specialuse] [pattern [base]]',
 		   'list mailboxes'],
 		server =>
 		  [\&_sc_server, '[-noauthenticate] [server]',
@@ -108,7 +108,7 @@ my %builtins = (exit =>
 		  [\&_sc_chdir, 'directory', 'change current directory'],
 		cd => 'chdir',
 		createmailbox =>
-		  [\&_sc_create, '[--partition partition] mailbox [partition]',
+		  [\&_sc_create, '[--partition partition] [--specialuse specialuse] mailbox [partition]',
 		   'create mailbox'],
 		create => 'createmailbox',
 		cm => 'createmailbox',
@@ -121,11 +121,15 @@ my %builtins = (exit =>
 		  [\&_sc_delete, 'mailbox [host]', 'delete mailbox'],
 		delete => 'deletemailbox',
 		dm => 'deletemailbox',
+		getmetadata =>
+		  [\&_sc_getmetadata, '[mailbox]',
+		   'display mailbox/server metadata'],
+		getmd => 'getmetadata',
 		info =>
 		  [\&_sc_info, '[mailbox]',
-		   'display mailbox/server metadata'],
+		   'display mailbox/server annotations'],
 		mboxcfg =>
-		  [\&_sc_mboxcfg, 'mailbox [comment|expire|news2mail|sieve|squat|/<explicit annotation>] value',
+		  [\&_sc_mboxcfg, '[--private] mailbox [comment|expire|news2mail|sieve|squat|/<explicit annotation>] value',
 		   'configure mailbox'],
 		mboxconfig => 'mboxcfg',
 		reconstruct =>
@@ -144,6 +148,10 @@ my %builtins = (exit =>
 		setinfo =>
 		  [\&_sc_setinfo, '[motd|comment|admin|shutdown|expire|squat] text',
 		   'set server metadata'],
+		setmetadata =>
+		  [\&_sc_setmetadata, 'mailbox [comment|expire|news2mail|sieve|squat|/<explicit annotation>] value',
+		   'set metadata to mailbox'],
+		setmd => 'setmetadata',
 		setquota =>
 		  [\&_sc_setquota,
 		   'mailbox resource value [resource value ...]',
@@ -600,27 +608,45 @@ sub _sc_exit {
 sub _sc_list {
   my ($cyrref, $name, $fh, $lfh, @argv) = @_;
   my $cmd = 'listmailbox';
-  my (@nargv, $opt);
+  my (@nargv, $opt, %opts, $subscribed);
   shift(@argv);
   while (defined ($opt = shift(@argv))) {
     # gack.  bloody tcl.
     last if $opt eq '--';
     if ($opt ne '' && '-subscribed' =~ /^\Q$opt/ || $opt eq '--subscribed') {
-      $cmd = 'listsubscribed';
+      $subscribed = 1;
+    } elsif ($opt ne '' && '-specialuse' =~ /^\Q$opt/ || $opt eq '--specialuse') {
+      $opts{'-sel-special-use'} = 1;
+    } elsif ($opt ne '' && '-recursivematch' =~ /^\Q$opt/ || $opt eq '--recursivematch') {
+      $opts{'-sel-recursivematch'} = 1;
     }
     elsif ($opt =~ /^-/) {
-      die "usage: listmailbox [-subscribed] [pattern [base]]\n";
+      die "usage: listmailbox [-subscribed] [-specialuse] [pattern [base]]\n";
     }
     else {
       push(@nargv, $opt);
       last;
     }
   }
+
+  if ($subscribed) {
+    if (scalar (keys %opts) > 0 ) {
+      # LIST + LIST-EXTENED
+      $opts{'-sel-subscribed'} = 1;
+    } else {
+      # LSUB
+      $cmd = 'listsubscribed';
+      # undef %opts;
+    }
+  }
+
   push(@nargv, @argv);
   if (@nargv > 2) {
-    die "usage: listmailbox [-subscribed] [pattern [base]]\n";
+    die "usage: listmailbox [-subscribed] [-specialuse] [pattern [base]]\n";
   }
   push(@nargv, '*') if !@nargv;
+  push(@nargv, undef) if scalar (@nargv) < 2; # no ref
+  push(@nargv, \%opts);
   if (!$cyrref || !$$cyrref) {
     die "listmailbox: no connection to server\n";
   }
@@ -635,6 +661,9 @@ sub _sc_list {
     if ($mbx->[1] ne '') {
       $l .= ' (' . $mbx->[1] . ')';
     }
+    if (defined ($mbx->[3])) {
+      $l .= ' (' . $mbx->[3] . ')';
+    }
     if (length($l) + 1 > $w) {
       $w = length($l) + 1;
     }
@@ -964,21 +993,29 @@ sub _sc_chdir {
 
 sub _sc_create {
   my ($cyrref, $name, $fh, $lfh, @argv) = @_;
-  my (@nargv, $opt, $part, $want);
+  my (@nargv, $opt, $part, $want, %opts);
   shift(@argv);
   while (defined ($opt = shift(@argv))) {
     if ($want) {
-      $part = $opt;
+      if ($want eq '-partition') {
+        $part = $opt;
+      } else {
+        $opts{$want} = $opt;
+      }
       $want = undef;
       next;
     }
     if ($opt ne '' && '-partition' =~ /^\Q$opt/ || $opt eq '--partition') {
-      $want = 1;
+      $want = '-partition';
+      next;
+    }
+    if ($opt ne '' && '-specialuse' =~ /^\Q$opt/ || $opt eq '--specialuse') {
+      $want = '-specialuse';
       next;
     }
     last if $opt eq '--';
     if ($opt =~ /^-/) {
-      die "usage: createmailbox [--partition partition] mailbox [partition]\n";
+      die "usage: createmailbox [--partition partition] [--specialuse specialuse] mailbox [partition]\n";
     }
     else {
       push(@nargv, $opt);
@@ -989,9 +1026,9 @@ sub _sc_create {
   if (!@nargv || @nargv > 2) {
     die "usage: createmailbox [--partition partition] mailbox [partition]\n";
   }
-  if (defined($part)) {
-      push(@nargv, $part)
-  }
+  push(@nargv, $part) if (defined ($part));
+  push(@nargv, undef) if (@nargv < 2);
+  push(@nargv, \%opts);
   if (!$cyrref || !$$cyrref) {
     die "createmailbox: no connection to server\n";
   }
@@ -1384,6 +1421,55 @@ sub _sc_info {
     return 1;
   }
 
+  foreach my $mailbox (sort keys %info) {
+      if($mailbox eq "") {
+        print "{Server Wide}\n";
+      } else {
+        print "{$mailbox}:\n";
+      }
+
+    my %attribname = ();
+    foreach my $attribname (sort keys %{$info{$mailbox}}) {
+      foreach my $attrib (sort keys %{$info{$mailbox}->{$attribname}}) {
+        if(!exists $attribname{$attribname}) {
+	  $attribname{$attribname} = 'x';
+	  print "  $attribname:\n";
+        }
+        $attrib =~ /([^\/]*)$/;
+        my $attrname = $1;
+
+        $lfh->[1]->print("    ", $attrname, ": ", $info{$mailbox}->{$attribname}->{$attrib}, "\n");
+      }
+    }
+  }
+  0;
+}
+
+sub _sc_getmetadata {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: getmetadata [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "info: no connection to server\n";
+  }
+  my %info = $$cyrref->getmetadata(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+
   # keep track of what mailboxes we've printed a header for already
   my %section = ();
   foreach my $attrib (sort keys %info) {
@@ -1403,6 +1489,30 @@ sub _sc_info {
 
     $lfh->[1]->print("  ", $attrname, ": ", $info{$attrib}, "\n");
   }
+}
+
+sub _sc_setmetadata {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: setmetadata mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|specialuse|squat|/<explicit metadata>] value\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (@nargv < 2) {
+    die "usage: setmetadata mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|specialuse|squat|/<explicit metadata>] value\n";
+  }
+  if (!$cyrref || !$$cyrref) {
+    die "setmetadata: no connection to server\n";
+  }
+  $$cyrref->setmetadata(@nargv) || die "setmetadata: " . $$cyrref->error . "\n";
   0;
 }
 
@@ -1462,12 +1572,14 @@ sub _sc_unsubscribe {
 
 sub _sc_mboxcfg {
   my ($cyrref, $name, $fh, $lfh, @argv) = @_;
-  my (@nargv, $opt);
+  my (@nargv, $opt, $private);
   shift(@argv);
   while (defined ($opt = shift(@argv))) {
     last if $opt eq '--';
-    if ($opt =~ /^-/) {
-      die "usage: mboxconfig mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|squat|/<explicit annotation>] value\n";
+    if ($opt ne '' && '-private' =~ /^\Q$opt/ || $opt eq '--private') {
+      $private = 1;
+    } elsif ($opt =~ /^-/) {
+      die "usage: mboxconfig [--private] mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|squat|/<explicit annotation>] value\n";
     }
     else {
       push(@nargv, $opt);
@@ -1476,7 +1588,10 @@ sub _sc_mboxcfg {
   }
   push(@nargv, @argv);
   if (@nargv < 2) {
-    die "usage: mboxconfig mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|squat|/<explicit annotation>] value\n";
+    die "usage: mboxconfig [--private] mailbox [comment|expire|news2mail|pop3showafter|sharedseen|sieve|squat|/<explicit annotation>] value\n";
+  }
+  if (defined ($private)) {
+    push(@nargv, $private);
   }
   if (!$cyrref || !$$cyrref) {
     die "mboxconfig: no connection to server\n";
@@ -1588,20 +1703,21 @@ authenticated once.
 Change directory.  A C<pwd> builtin is not provided, but the default command
 action will run C<pwd> from a shell if invoked.
 
-=item C<createmailbox> [C<--partition> I<partition>] I<mailbox>
+=item C<createmailbox> [C<--partition> I<partition>] [C<--specialuse> I<specialuse>] I<mailbox>
 
-=item C<createmailbox> I<mailbox> I<partition>
+=item C<createmailbox> [C<--specialuse> I<specialuse>] I<mailbox> I<partition>
 
-=item C<create> [C<--partition> I<partition>] I<mailbox>
+=item C<create> [C<--partition> I<partition>] [C<--specialuse> I<specialuse>] I<mailbox>
 
-=item C<create> I<mailbox> I<partition>
+=item C<create> [C<--specialuse> I<specialuse>] I<mailbox> I<partition>
 
-=item C<cm> [C<--partition> I<partition>] I<mailbox>
+=item C<cm> [C<--partition> I<partition>] [C<--specialuse> I<specialuse>] I<mailbox>
 
-=item C<cm> I<mailbox> I<partition>
+=item C<cm> [C<--specialuse> I<specialuse>] I<mailbox> I<partition>
 
 Create a mailbox on the default or a specified partition.  Both old-style
 and getopt-style usages are accepted (combining them will produce an error).
+Optionally assign a special use to the mailbox.
 
 =item C<deleteaclmailbox> I<mailbox> I<id> [...]
 
@@ -1658,15 +1774,15 @@ Display the mailbox/server metadata.
 
 List ACLs on the specified mailbox.
 
-=item C<listmailbox> [C<--subscribed>] [I<pattern> [I<reference>]]
+=item C<listmailbox> [C<--subscribed>] [C<--specialuse>] [I<pattern> [I<reference>]]
 
-=item C<list> [C<--subscribed>] [I<pattern> [I<reference>]]
+=item C<list> [C<--subscribed>] [C<--specialuse>] [I<pattern> [I<reference>]]
 
-=item C<lm> [C<--subscribed>] [I<pattern> [I<reference>]]
+=item C<lm> [C<--subscribed>] [C<--specialuse>] [I<pattern> [I<reference>]]
 
-List all, or all subscribed, mailboxes matching the specified pattern.
-The pattern may have embedded wildcards C<'*'> or C<'%'>, which match
-anything or anything except the separator character, respectively.
+List all, or all subscribed or special-use, mailboxes matching the specified
+pattern.  The pattern may have embedded wildcards C<'*'> or C<'%'>, which
+match anything or anything except the separator character, respectively.
 
 Mailboxes returned will be relative to the specified reference if one
 is specified.  This allows a mailbox list to be limited to a particular
@@ -1693,12 +1809,14 @@ find the quota root for a mailbox.
 
 show quota roots and quotas for mailbox
 
-=item C<mboxconfig> I<mailbox> I<attribute> I<value>
+=item C<mboxconfig> [C<--private>] I<mailbox> I<attribute> I<value>
 
-=item C<mboxcfg> I<mailbox> I<attribute> I<value>
+=item C<mboxcfg> [C<--private>] I<mailbox> I<attribute> I<value>
 
-Set mailbox metadata.  A value of "none" will remove the attribute.
-The currently supported attributes are: 
+Set mailbox metadata, optionally set the private instead of the shared
+version of the metadata. A value of "none" will remove the attribute.
+
+The currently supported attributes are:
 
 =over 4
 
