@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 3.0.2.  */
+/* A Bison parser, made by GNU Bison 2.5.  */
 
 /* Bison interface for Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
-
+   
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
+   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,121 +26,113 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_SIEVE_SIEVE_SIEVE_H_INCLUDED
-# define YY_SIEVE_SIEVE_SIEVE_H_INCLUDED
-/* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int sievedebug;
-#endif
 
-/* Token type.  */
+/* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-  enum yytokentype
-  {
-    NUMBER = 258,
-    STRING = 259,
-    IF = 260,
-    ELSIF = 261,
-    ELSE = 262,
-    REJCT = 263,
-    FILEINTO = 264,
-    REDIRECT = 265,
-    KEEP = 266,
-    STOP = 267,
-    DISCARD = 268,
-    VACATION = 269,
-    REQUIRE = 270,
-    SETFLAG = 271,
-    ADDFLAG = 272,
-    REMOVEFLAG = 273,
-    MARK = 274,
-    UNMARK = 275,
-    HASFLAG = 276,
-    FLAGS = 277,
-    NOTIFY = 278,
-    DENOTIFY = 279,
-    ANYOF = 280,
-    ALLOF = 281,
-    EXISTS = 282,
-    SFALSE = 283,
-    STRUE = 284,
-    HEADER = 285,
-    NOT = 286,
-    SIZE = 287,
-    ADDRESS = 288,
-    ENVELOPE = 289,
-    BODY = 290,
-    COMPARATOR = 291,
-    IS = 292,
-    CONTAINS = 293,
-    MATCHES = 294,
-    REGEX = 295,
-    COUNT = 296,
-    VALUE = 297,
-    OVER = 298,
-    UNDER = 299,
-    GT = 300,
-    GE = 301,
-    LT = 302,
-    LE = 303,
-    EQ = 304,
-    NE = 305,
-    ALL = 306,
-    LOCALPART = 307,
-    DOMAIN = 308,
-    USER = 309,
-    DETAIL = 310,
-    RAW = 311,
-    TEXT = 312,
-    CONTENT = 313,
-    DAYS = 314,
-    ADDRESSES = 315,
-    SUBJECT = 316,
-    FROM = 317,
-    HANDLE = 318,
-    MIME = 319,
-    SECONDS = 320,
-    METHOD = 321,
-    ID = 322,
-    OPTIONS = 323,
-    LOW = 324,
-    NORMAL = 325,
-    HIGH = 326,
-    ANY = 327,
-    MESSAGE = 328,
-    INCLUDE = 329,
-    PERSONAL = 330,
-    GLOBAL = 331,
-    RETURN = 332,
-    OPTIONAL = 333,
-    ONCE = 334,
-    COPY = 335,
-    DATE = 336,
-    CURRENTDATE = 337,
-    INDEX = 338,
-    LAST = 339,
-    ZONE = 340,
-    ORIGINALZONE = 341,
-    YEAR = 342,
-    MONTH = 343,
-    DAY = 344,
-    JULIAN = 345,
-    HOUR = 346,
-    MINUTE = 347,
-    SECOND = 348,
-    TIME = 349,
-    ISO8601 = 350,
-    STD11 = 351,
-    WEEKDAY = 352
-  };
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     NUMBER = 258,
+     STRING = 259,
+     IF = 260,
+     ELSIF = 261,
+     ELSE = 262,
+     REJCT = 263,
+     FILEINTO = 264,
+     REDIRECT = 265,
+     KEEP = 266,
+     STOP = 267,
+     DISCARD = 268,
+     VACATION = 269,
+     REQUIRE = 270,
+     SETFLAG = 271,
+     ADDFLAG = 272,
+     REMOVEFLAG = 273,
+     MARK = 274,
+     UNMARK = 275,
+     HASFLAG = 276,
+     FLAGS = 277,
+     NOTIFY = 278,
+     DENOTIFY = 279,
+     ANYOF = 280,
+     ALLOF = 281,
+     EXISTS = 282,
+     SFALSE = 283,
+     STRUE = 284,
+     HEADER = 285,
+     NOT = 286,
+     SIZE = 287,
+     ADDRESS = 288,
+     ENVELOPE = 289,
+     BODY = 290,
+     COMPARATOR = 291,
+     IS = 292,
+     CONTAINS = 293,
+     MATCHES = 294,
+     REGEX = 295,
+     COUNT = 296,
+     VALUE = 297,
+     OVER = 298,
+     UNDER = 299,
+     GT = 300,
+     GE = 301,
+     LT = 302,
+     LE = 303,
+     EQ = 304,
+     NE = 305,
+     ALL = 306,
+     LOCALPART = 307,
+     DOMAIN = 308,
+     USER = 309,
+     DETAIL = 310,
+     RAW = 311,
+     TEXT = 312,
+     CONTENT = 313,
+     DAYS = 314,
+     ADDRESSES = 315,
+     SUBJECT = 316,
+     FROM = 317,
+     HANDLE = 318,
+     MIME = 319,
+     SECONDS = 320,
+     METHOD = 321,
+     ID = 322,
+     OPTIONS = 323,
+     LOW = 324,
+     NORMAL = 325,
+     HIGH = 326,
+     ANY = 327,
+     MESSAGE = 328,
+     INCLUDE = 329,
+     PERSONAL = 330,
+     GLOBAL = 331,
+     RETURN = 332,
+     OPTIONAL = 333,
+     ONCE = 334,
+     COPY = 335,
+     DATE = 336,
+     CURRENTDATE = 337,
+     INDEX = 338,
+     LAST = 339,
+     ZONE = 340,
+     ORIGINALZONE = 341,
+     YEAR = 342,
+     MONTH = 343,
+     DAY = 344,
+     JULIAN = 345,
+     HOUR = 346,
+     MINUTE = 347,
+     SECOND = 348,
+     TIME = 349,
+     ISO8601 = 350,
+     STD11 = 351,
+     WEEKDAY = 352
+   };
 #endif
 /* Tokens.  */
 #define NUMBER 258
@@ -239,12 +231,15 @@ extern int sievedebug;
 #define STD11 351
 #define WEEKDAY 352
 
-/* Value type.  */
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE YYSTYPE;
-union YYSTYPE
+typedef union YYSTYPE
 {
-#line 213 "sieve/sieve.y" /* yacc.c:1909  */
+
+/* Line 2068 of yacc.c  */
+#line 213 "sieve.y"
 
     int nval;
     char *sval;
@@ -262,14 +257,16 @@ union YYSTYPE
     struct dttags *dttag;
     struct ftags *ftag;
 
-#line 266 "sieve/sieve.h" /* yacc.c:1909  */
-};
+
+
+/* Line 2068 of yacc.c  */
+#line 264 "sieve/sieve.h"
+} YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
 
 
-int sieveparse (sieve_script_t *parse_script);
 
-#endif /* !YY_SIEVE_SIEVE_SIEVE_H_INCLUDED  */
